/**
Provides a web worker process for Michel L.'s java propellerload. 
**/

// ID for console output
var consoleID = 0;
    
/**
 * Send progress update to parent process.
 *
 * @method reportProgress
 * @param  {Number} value Percentage of progress so far.
 */
function reportProgress (value) {
    self.postMessage({msgType: 'progress', 'id': consoleID, 'value': value });
}

/**
 * Send a log message to the parent process.
 *
 * @method log
 * @param  {String} s The log message text.
 */
function log (s) {
    self.postMessage({msgType: 'log', 'id': consoleID, string: s });
}

function runPropellerload(data) {
    var xhr = new XMLHttpRequest();
    try {
        reportProgress (0);
        xhr.open('PUT', '/webapp/propeller.action?action=' + data.msgType, true);
        xhr.onload = function(e) {
            var output = typeof xhr.response == 'string' ? JSON.parse(xhr.response) : xhr.response;
            reportProgress (100);
            log ('w propeller-web-loader - done ');
            log ('w success:' + output.succes + ' code:' + output.code);
            log ('w message:\n' + output.message);
            terminate();
        };
        xhr.onreadystatechange = function() {
            log ('w propeller-web-loader - ' + xhr.readyState);
        };
        xhr.send(data.buffer);
    } catch (e) {
        log ('w Exception propeller-web-loader: ' + e );
        log ('w Exception propeller-web-loader Stack: ' + e.stack );
        reportProgress (100);
        terminate();
    }

}

function terminate() {
    self.postMessage({msgType: 'terminating', 'id': consoleID});
    self.close();    // Terminates the worker.
}

self.addEventListener('message', function(e) {
    var data = e.data;
    consoleID = data.id;
    switch (data.msgType) {
        case 'LOAD_EEPROM_BUF':
            log ('w Propellerload Worker started - consoleID: ' + data.id);
            runPropellerload(data);
            break;            
        case 'LOAD_RAM_BUF':
            log ('w Propellerload Worker started - consoleID: ' + data.id);
            runPropellerload(data);
            break;
        case 'terminate':
            reportProgress (100);
            terminate();
            break;
        default:
            self.postMessage({msgType: "unknown", 'id': consoleID});
    }
}, false);
